/****** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is XpcomViewer code.
 *
 * The Initial Developer of the Original Code is
 * Ondrej Donek.
 * Portions created by Initial Developer are Copyright (C) 2005-2006
 * Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *  - Ondrej Donek, <ondrejd@gmail.com>, original author
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the LGPL or the GPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 ****** END LICENSE BLOCK *****/


/**
 * Class that serves all what is needed for working with notes
 * datasource and its file.
 * Note: Datasource is not initialized!
 */
var gNotesDataSource = new newDataSource ();

/**
 * Add new note into notes container, save datasource
 * and refresh allNotes-menu.
 *
 * @param   string  Name of item we attach note to
 * @param   string  Title of note
 * @param   string  Text of note
 * @return  boolean Returns true if operation successfull
 */
gNotesDataSource.add = function (aItemName, aNoteTitle, aNoteText) {
  try {
		var newNote = kRdfSrv.GetResource (this._uriBase + ":" +
		    (Math.round(Math.random ()*64*32*8)));

		this._ds.Assert (newNote, this._getArc ("target"), kRdfSrv.GetLiteral (aItemName), true);
		this._ds.Assert (newNote, this._getArc ("title"), kRdfSrv.GetLiteral (aNoteTitle), true);
		this._ds.Assert (newNote, this._getArc ("text"), kRdfSrv.GetLiteral (aNoteText), true);
		this._rootCont.AppendElement (newNote);
		this.save ();

		gStatusbar.updateLabel ("Note successfully attached to [" + aItemName + "].");

	  return true;
	} catch (e) {
	  Components.utils.reportError (e);
	  return false;
	}
};

/**
 * Returns true if item with given name has attached note
 * or false if not.
 *
 * @return boolean
 */
gNotesDataSource.hasItemNote = function (aItemName)
{
  try {
		var elms = this._rootCont.GetElements ();

		while (elms.hasMoreElements ()) {
		  var elm = elms.getNext ();
		  var target = this._ds.GetTarget (elm, this._getArc ("target"), true);

		  if (target instanceof Components.interfaces.nsIRDFLiteral) {
	      if (target.Value == aItemName) return true;
	    }
		}

		return false;
	} catch (e) {
	  Components.utils.reportError (e);
	  return false;
	}
};

/**
 * Returns array with all names of note's targets.
 *
 * @param  int   Max.limit (if <=0 ommitted)
 * @return array
 */
gNotesDataSource.getAllNotes = function (aMaxLimit)
{
  var notes = [];

  try {
    var elms = this._rootCont.GetElements ();
    var counter = 0;

    while (elms.hasMoreElements ()) {
      var elm = elms.getNext ();
      var target = this._ds.GetTarget (elm, this._getArc ("title"), true);

      if (target instanceof Components.interfaces.nsIRDFLiteral)
        notes.push (target.Value);

      if (aMaxLimit > 0 && aMaxLimit == counter) return notes;

      counter++;
    }
  } catch (e) {
    Components.utils.reportError (e);
  }

  return notes;
};

/**
 * Remove note
 *
 * @param   nsIRDFResource  Resource to which is note appended
 */
gNotesDataSource.remove = function (aItemName)
{
  try {
		var elms = this._rootCont.GetElements ();
    var changed = false;

		while (elms.hasMoreElements ()) {
		  var elm = elms.getNext ();
		  var target = this._ds.GetTarget (elm, this._getArc ("target"), true);
		  var label = "";

		  if (target instanceof Components.interfaces.nsIRDFLiteral)
		    label = target.Value;

      if (label == aItemName) {
        Components.utils.reportError (label);
        this._rootCont.RemoveElement (elm, true);
        changed = true;
      }
		}

		if (changed) {
			writeDataSourceToFile (this._dsFile, this._ds);
			this.refreshElements ();
		  gStatusbar.updateLabel ("Item [" + aItemName + "] successfully deleted.");
		} else {
		  gStatusbar.updateLabel ("Item [" + aItemName + "] not founded!");
		}

		return true;
	} catch (e) {
	  Components.utils.reportError (e);
	  return false;
	}
};

/**
 * Remove all notes
 */
gNotesDataSource.removeAll = function ()
{
	this._ds = null;
	this.createBlankDataSource ();
	this.saveDataSourceToFile ();
	this._rootCont = Components.classes["@mozilla.org/rdf/container;1"].
	    createInstance (Components.interfaces.nsIRDFContainer);
	this._rootCont.Init (this._ds, this._resBase);
};
